using System;
using System.Numerics;

using Microsoft.UI.Composition;
using Microsoft.UI.Xaml;
using Microsoft.UI.Xaml.Controls;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainPage()

        /// <summary>
        /// 
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            this.scrollView.ScrollAnimationStarting += scrollView_ScrollAnimationStarting;
            this.scrollButton.Click                 += scrollButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region ũ  ũ ִϸ̼ ۽ óϱ - scrollView_ScrollAnimationStarting(sender, e)

        /// <summary>
        /// ũ  ũ ִϸ̼ ۽ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void scrollView_ScrollAnimationStarting(ScrollView sender, ScrollingScrollAnimationStartingEventArgs e)
        {
            Vector3KeyFrameAnimation animation1 = e.Animation as Vector3KeyFrameAnimation;

            if(animation1 != null)
            {
                if(this.vcerticalAnimationComboBox.SelectedIndex == 0)
                {
                    animation1.Duration = TimeSpan.FromMilliseconds(this.animationDurationNumberBox.Value);
                }
                else
                {
                    double targetVerticalOffset = GetTargetVerticalOffset();

                    float targetVerticalPosition = (float)targetVerticalOffset;

                    Vector3KeyFrameAnimation animation2 = animation1.Compositor.CreateVector3KeyFrameAnimation();

                    if(this.vcerticalAnimationComboBox.SelectedIndex == 1)
                    {
                        float deltaVerticalPosition = 0.1f * (float)(targetVerticalOffset - this.scrollView.VerticalOffset);

                        for(int step = 0; step < 3; step++)
                        {
                            animation2.InsertKeyFrame
                            (
                                1.0f - (0.4f / (float)Math.Pow(2, step)),
                                new Vector3
                                (
                                    (float)this.scrollView.HorizontalOffset,
                                    targetVerticalPosition + deltaVerticalPosition,
                                    0.0f
                                )
                            );

                            deltaVerticalPosition /= -2.0f;
                        }

                        animation2.InsertKeyFrame(1.0f, new Vector3((float)this.scrollView.HorizontalOffset, targetVerticalPosition, 0.0f));
                    }
                    else
                    {
                        float deltaVerticalPosition = (float)(targetVerticalOffset - this.scrollView.VerticalOffset);

                        CubicBezierEasingFunction startCubicBezierEasingFunction = animation1.Compositor.CreateCubicBezierEasingFunction
                        (
                            new Vector2(1.0f, 0.0f),
                            new Vector2(1.0f, 0.0f)
                        );

                        StepEasingFunction stepEasingFunction = animation1.Compositor.CreateStepEasingFunction(1);

                        CubicBezierEasingFunction endCubicBezierEasingFunction = animation1.Compositor.CreateCubicBezierEasingFunction
                        (
                            new Vector2(0.0f, 1.0f),
                            new Vector2(0.0f, 1.0f)
                        );

                        animation2.InsertKeyFrame
                        (
                            0.499999f,
                            new Vector3
                            (
                                (float)this.scrollView.HorizontalOffset,
                                targetVerticalPosition - 0.9f * deltaVerticalPosition,
                                0.0f
                            ),
                            startCubicBezierEasingFunction
                        );

                        animation2.InsertKeyFrame
                        (
                            0.5f,
                            new Vector3
                            (
                                (float)this.scrollView.HorizontalOffset,
                                targetVerticalPosition - 0.1f * deltaVerticalPosition,
                                0.0f
                            ),
                            stepEasingFunction
                        );

                        animation2.InsertKeyFrame
                        (
                            1.0f,
                            new Vector3
                            (
                                (float)this.scrollView.HorizontalOffset,
                                targetVerticalPosition,
                                0.0f
                            ),
                            endCubicBezierEasingFunction
                        );
                    }

                    animation2.Duration = TimeSpan.FromMilliseconds(this.animationDurationNumberBox.Value);

                    e.Animation = animation2;
                }
            }
        }

        #endregion
        #region Scroll with animation ư Ŭ óϱ - scrollButton_Click(sender, e)

        /// <summary>
        /// Scroll with animation ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void scrollButton_Click(object sender, RoutedEventArgs e)
        {
            if(this.scrollView != null)
            {
                this.scrollView.ScrollTo
                (
                    this.scrollView.HorizontalOffset,
                    GetTargetVerticalOffset(),
                    new ScrollingScrollOptions(ScrollingAnimationMode.Enabled, ScrollingSnapPointsMode.Ignore)
                );
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region Ÿ   ϱ - GetTargetVerticalOffset()

        /// <summary>
        /// Ÿ   ϱ
        /// </summary>
        /// <returns>Ÿ   ϱ</returns>
        private double GetTargetVerticalOffset()
        {
            if(this.scrollView.VerticalOffset > this.scrollView.ScrollableHeight / 2.0)
            {
                return this.scrollView.ScrollableHeight / 5.0;
            }
            else
            {
                return 4.0 * this.scrollView.ScrollableHeight / 5.0;
            }
        }

        #endregion
    }
}